#!/usr/bin/python2.4
# -*- coding: utf-8 -*-

import re
import sys
from cStringIO import StringIO

import pycurl

from moio.plugins.Sender import Sender
from moio.plugins.CaptchaDecoder import CaptchaDecoder
from moio.ConnectionManager import ConnectionManager
from moio.errors.SiteCustomError import SiteCustomError
from moio.errors.CaptchaError import CaptchaError
from moio.errors.SiteConnectionError import SiteConnectionError
from moio.errors.SiteAuthError import SiteAuthError
from moio.errors.SenderError import SenderError

class Vodafonemms(Sender):
    """Permette di spedire MMS testuali dal sito vodafone.it"""
    
    maxLength = 500
    """Lunghezza massima del messaggio singolo inviabile da questo sito."""

    requiresRegistration = True
    """Questo plugin richiede nome utente e password?"""
    
    def isAvailable(self):
        """Ritorna true se questo plugin è utilizzabile."""
        return True
    
    def sendOne(self, number, text, username = None, password = None):
        """Spedisce un SMS con soli caratteri ASCII e di lunghezza massima maxLength
        con le credenziali specificate, supponendo Internet raggiungibile.
        """
        try:
            #Costruisco un nuovo oggetto Curl e lo inizializzo
            c = self.connectionManager.getCurl()
            c.setopt(pycurl.POST, False)
            
            if number[:3] == "+39":
                number = number[3:]
            elif number[0]=="+":
                raise SiteCustomError(self.__class__.__name__,
                u"Questo sito permette di inviare SMS solo verso cellulari italiani.")
        
            #Visito la pagina iniziale
            saver = StringIO()
            c.setopt(pycurl.WRITEFUNCTION, saver.write)
            c.setopt(pycurl.URL, "http://www.vodafone.it/190/trilogy/jsp/home.do")
            c.perform()
            self.checkForErrors(saver.getvalue())
                        
            #Sono già autenticato?
            if(re.search("Ciao: <b><!-- TY_DISP -->", saver.getvalue()) is None):
                #No, ammazzo i vecchi cookie e mi riautentico
                self.connectionManager.forgetCookiesFromDomain("190.it")
                self.connectionManager.forgetCookiesFromDomain("vodafone.it")
                saver = StringIO()
                c.setopt(pycurl.WRITEFUNCTION, saver.write)
                c.setopt(pycurl.URL, "https://www.vodafone.it/190/trilogy/jsp/login.do")
                postFields = {}
                postFields["username"] = username
                postFields["password"] = password
                c.setopt(pycurl.POST, True)
                c.setopt(pycurl.POSTFIELDS, self.codingManager.urlEncode(postFields))
                c.perform()

                self.checkForErrors(saver.getvalue())
                if (re.search("Ciao: <b><!-- TY_DISP -->", saver.getvalue()) is None):
                    raise SiteAuthError(self.__class__.__name__)
                                  
            #Visito la pubblicità obbligatoria
            c.setopt(pycurl.WRITEFUNCTION, self.doNothing)
            c.setopt(pycurl.POST, False)
            c.setopt(pycurl.URL,
                "http://www.areaprivati.vodafone.it/190/trilogy/jsp/dispatcher.do?ty_key=mmlsendmms&tk=9609,c")
            c.perform()

            saver = StringIO()
            headerSaver = StringIO()
            c.setopt(pycurl.WRITEFUNCTION, saver.write)
            c.setopt(pycurl.HEADERFUNCTION, headerSaver.write)
            c.setopt(pycurl.URL,
                "http://mmsviaweb.net.vodafone.it/WebComposer/web/mms.jsp")
            c.perform()
            
            authData = re.search('AuthData=(%[^\n]+)\n', headerSaver.getvalue())
            if (authData is None):
                raise SenderError(self.__class__.__name__)
           
            authData = authData.group(1)
            postFields = {}
            postFields["AuthData"] = authData
            postString = self.codingManager.urlEncode(postFields);
            postSize = len(postString)
            
            saver = StringIO()
            c.setopt(pycurl.WRITEFUNCTION, saver.write)
            c.setopt(pycurl.POST, True)
            c.setopt(pycurl.POSTFIELDS, postString)
            c.setopt(pycurl.POSTFIELDSIZE, postSize)
            c.setopt(pycurl.URL,
                "http://mmsviaweb.net.vodafoneomnitel.it/WebComposer/web/mms.jsp")
            c.perform()
            
                                    
            #Spedisco l'MMS
            saver = StringIO()
            c.setopt(pycurl.WRITEFUNCTION, saver.write)
            c.setopt(pycurl.POST, False)
            c.setopt(pycurl.URL,
                "http://mmsviaweb.net.vodafoneomnitel.it/WebComposer/web/popup.jsp?Caso=7")
            c.perform()
            
            if (re.search(
                "Ti ricordiamo che puoi inviare SMS via Web solo a numeri di cellulare Vodafone",
                  saver.getvalue()) is not None or
                re.search(
                "Il numero di telefono del destinatario del messaggio non e' valido",
                saver.getvalue()) is not None):
                raise SiteCustomError(self.__class__.__name__, u"Questo sito permette di inviare " +
                        u"SMS solo ai cellulari Vodafone.")
            
            postFields = {}
            postFields["recipient"] = "+39" + number
            postFields["subjecttosend"] = "Messaggio da MoioSMS"
            postFields["SmilName"] = ""
            postFields["TextName"] = text
            postFields["ImageName"] = ""
            postFields["AudioName"] = ""
            postFields["nextPage"] = "/web/servletresult.html"

            c.setopt(pycurl.POST, True)
            c.setopt(pycurl.POSTFIELDS,
                self.codingManager.urlEncode(postFields))
    
            #Confermo l'invio
            saver = StringIO()
            c.setopt(pycurl.WRITEFUNCTION, saver.write)
            c.setopt(pycurl.URL,
                "http://mmsviaweb.net.vodafoneomnitel.it/WebComposer/web/elaborapop.jsp")
            c.perform()

            if (re.search("Il tuo messaggio &egrave; stato inviato", saver.getvalue()) is None):
                raise SenderError(self.__class__.__name__)
            
        except pycurl.error, e:
            errno, msg = e
            raise SiteConnectionError(self.__class__.__name__, self.codingManager.iso88591ToUnicode(msg))

    def checkForErrors(self, page):
        """Solleva un'eccezione se la pagina contiene una segnalazione d'errore dal sito."""
        if(re.search("intervento di manutenzione", page) is not None):
            raise SiteCustomError(self.__class__.__name__, u"Il sito è in manutenzione, riprova più tardi.")
        if(re.search("numero massimo di accessi", page) is not None):
            raise SiteCustomError(self.__class__.__name__, u"Ci sono troppi utenti sul sito, riprova più tardi.")
